#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "cuda.h"

#define SIZE                (1024)
#define THREADS_PER_BLOCK   (256)

__global__ void dotprod(float *a, float *b, float *c, int n)
{
    int tid = blockDim.x * blockIdx.x + threadIdx.x;
    while (tid < n)
    {
        c[tid] = a[tid] * b[tid];
        tid += blockDim.x * gridDim.x;
    }
}

int main(void)
{
    float *h_a, *h_b, *h_c;
    float *d_a, *d_b, *d_c;
    
	// Rezervacija pomnilnika - gostitelj
	h_a = (float *)malloc(SIZE * sizeof(float));
	h_b = (float *)malloc(SIZE * sizeof(float));
	h_c = (float *)malloc(SIZE * sizeof(float));

	// Inicializacija vektorjev
    srand(time(NULL));
	for (int i = 0; i < SIZE; i++)
	{
		h_a[i] = (double)rand()/RAND_MAX;
		h_b[i] = (double)rand()/RAND_MAX;;
	}

    // Rezervacija pomnilnika - naprava
    cudaMalloc((void **)&d_a, SIZE * sizeof(float));
    cudaMalloc((void **)&d_b, SIZE * sizeof(float));
    cudaMalloc((void **)&d_c, SIZE * sizeof(float));

    // prenos podatkov na napravo
    cudaMemcpy(d_a, h_a, SIZE * sizeof(float), cudaMemcpyHostToDevice);
    cudaMemcpy(d_b, h_b, SIZE * sizeof(float), cudaMemcpyHostToDevice);

	// Racunanje
    int blockspergrid = (SIZE-1)/THREADS_PER_BLOCK + 1;
    dotprod<<<blockspergrid,THREADS_PER_BLOCK>>>(d_a, d_b, d_c, SIZE);
    cudaError_t err = cudaGetLastError();
    if (err != cudaSuccess) 
    {
        printf("cudaGetDeviceCount error %d\n-> %s\n", err, cudaGetErrorString(err));
        exit(EXIT_FAILURE);
    }


    // prenos podatkov iz naprave
    cudaMemcpy(h_c, d_c, SIZE * sizeof(float), cudaMemcpyDeviceToHost);

    // sproscanje pomnilnika
    cudaFree(d_a);
    cudaFree(d_b);
    cudaFree(d_c);

    float dotProdCPU = 0.0;
    for(int i =0; i< SIZE; i++)
        dotProdCPU += h_a[i] * h_b[i];
    float dotProdGPU = 0.0;
    for(int i =0; i< SIZE; i++)
        dotProdGPU += h_c[i];

	// Prikaz rezultatov
    printf("dotprod(CPU): %f\ndotprod(GPU): %f\n", dotProdCPU, dotProdGPU);

	free(h_a);
	free(h_b);
	free(h_c);

	return 0;
}
